@@ -159,22 +159,34 @@ hxmc_t *pmt_vol_to_dev(const struct vol *vol)
 	switch (vol->type) {
 	case CMD_SMBMOUNT:
 	case CMD_CIFSMOUNT:
-		ret = HXmc_strinit("//");
-		HXmc_strcat(&ret, vol->server);
-		HXmc_strcat(&ret, "/");
-		HXmc_strcat(&ret, vol->volume);
+		if (vol->server != NULL) {
+			ret = HXmc_strinit("//");
+			HXmc_strcat(&ret, vol->server);
+			HXmc_strcat(&ret, "/");
+			HXmc_strcat(&ret, vol->volume);
+		} else {
+			ret = HXmc_strinit(vol->volume);
+		}
 		break;
 
 	case CMD_NCPMOUNT:
-		ret = HXmc_strinit(vol->server);
-		HXmc_strcat(&ret, "/");
-		HXmc_strcat(&ret, kvplist_get(&vol->options, "user"));
+		if (vol->server != NULL) {
+			ret = HXmc_strinit(vol->server);
+			HXmc_strcat(&ret, "/");
+			HXmc_strcat(&ret, kvplist_get(&vol->options, "user"));
+		} else {
+			ret = HXmc_strinit(vol->volume);
+		}
 		break;
 
 	case CMD_NFSMOUNT:
-		ret = HXmc_strinit(vol->server);
-		HXmc_strcat(&ret, ":");
-		HXmc_strcat(&ret, vol->volume);
+		if (vol->server != NULL) {
+			ret = HXmc_strinit(vol->server);
+			HXmc_strcat(&ret, ":");
+			HXmc_strcat(&ret, vol->volume);
+		} else {
+			ret = HXmc_strinit(vol->volume);
+		}
 		break;
 
 	default:
@@ -638,6 +650,13 @@ int mount_op(mount_op_fn_t *mnt, const struct config *config,
 	if ((vinfo = HXformat_init()) == NULL)
 		return 0;
 
+	HXmc_free(vpt->combopath);
+	vpt->combopath = pmt_vol_to_dev(vpt);
+	if (vpt->combopath == NULL) {
+		l0g("vol_to_dev: %s\n", strerror(errno));
+		return 0;
+	}
+
 	if (realpath(vpt->mountpoint, real_mpt) == NULL) {
 		w4rn("Could not get realpath of %s: %s\n",
 		     vpt->mountpoint, strerror(errno));
@@ -650,6 +669,7 @@ int mount_op(mount_op_fn_t *mnt, const struct config *config,
 	format_add(vinfo, "MNTPT",    vpt->mountpoint);
 	format_add(vinfo, "FSTYPE",   vpt->fstype);
 	format_add(vinfo, "VOLUME",   vpt->volume);
+	format_add(vinfo, "COMBOPATH", vpt->combopath);
 	format_add(vinfo, "SERVER",   vpt->server);
 	format_add(vinfo, "USER",     vpt->user);
 	format_add(vinfo, "CIPHER",   vpt->cipher);
