@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2008 - 2010
+ *	Copyright © Jan Engelhardt, 2008-2011
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -43,7 +43,8 @@
  * @remount:		issue a remount
  */
 struct mount_options {
-	const char *object, *container, *mountpoint, *fstype;
+	hxmc_t *object, *container, *mountpoint;
+	const char *fstype;
 	const char *dmcrypt_cipher, *dmcrypt_hash;
 	const char *fsk_hash, *fsk_cipher, *fsk_file;
 	hxmc_t *fsk_password, *extra_opts, *crypto_device;
@@ -64,57 +65,11 @@ struct mount_options {
  * @blkdev:		@container is a block device
  */
 struct umount_options {
-	const char *object;
+	hxmc_t *object;
 	unsigned int no_update, ro_fallback;
 	bool is_cont, blkdev;
 };
 
-static const char *const mtab_file  = "/etc/mtab";
-static const char *const kmtab_file = "/proc/mounts";
-
-static hxmc_t *readlinkf(const char *file)
-{
-	static const unsigned int bufsize = 256;
-	const char *const args[] = {"readlink", "-fn", file, NULL};
-	struct HXproc proc;
-	ssize_t readret;
-	char *buf;
-	int ret;
-
-	if ((buf = malloc(bufsize)) == NULL) {
-		fprintf(stderr, "malloc: %s\n", strerror(errno));
-		return NULL;
-	}
-	arglist_llog(args);
-	memset(&proc, 0, sizeof(proc));
-	proc.p_flags = HXPROC_VERBOSE | HXPROC_STDOUT;
-	if ((ret = HXproc_run_async(args, &proc)) <= 0) {
-		fprintf(stderr, "spawn_startl readlink: %s\n", strerror(-ret));
-		free(buf);
-		return NULL;
-	}
-
-	readret = read(proc.p_stdout, buf, bufsize);
-	if (readret < 0) {
-		fprintf(stderr, "read: %s\n", strerror(errno));
-		goto out;
-	} else if (readret >= bufsize) {
-		fprintf(stderr, "Hm, path too long for us\n");
-		goto out;
-	}
-
-	buf[readret] = '\0';
-	close(proc.p_stdout);
-	HXproc_wait(&proc);
-	return buf;
-
- out:
-	free(buf);
-	close(proc.p_stdout);
-	HXproc_wait(&proc);
-	return NULL;
-}
-
 static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 {
 	struct mount_options *mo = cbi->current->uptr;
@@ -185,9 +140,9 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 		if (strcmp(key, "remount") == 0)
 			mo->remount = true;
 		else if (strcmp(key, "ro") == 0)
-			mo->readonly = true;
+			mo->readonly = LOSETUP_RO;
 		else if (strcmp(key, "rw") == 0)
-			mo->readonly = false;
+			mo->readonly = LOSETUP_RW;
 	}
 
 	if (*passthru != '\0') {
@@ -242,7 +197,14 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 			return false;
 		}
 
-		opt->object = readlinkf((*argv)[1]);
+		/* Only absolute paths should be in mtab. */
+		ret = HX_realpath(&opt->object, (*argv)[1],
+		      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+		if (ret < 0) {
+			fprintf(stderr, "realpath %s: %s\n",
+			        (*argv)[1], strerror(-ret));
+			return false;
+		}
 		if (stat(opt->object, &sb) < 0) {
 			/* If it does not exist, it cannot be the container. */
 			opt->is_cont = false;
@@ -273,8 +235,20 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		return false;
 	}
 
-	opt->container  = readlinkf((*argv)[1]);
-	opt->mountpoint = readlinkf((*argv)[2]);
+	ret = HX_realpath(&opt->container, (*argv)[1],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[1], strerror(-ret));
+		return false;
+	}
+	ret = HX_realpath(&opt->mountpoint, (*argv)[2],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[2], strerror(-ret));
+		return false;
+	}
 
 	if (stat(opt->mountpoint, &sb) < 0) {
 		fprintf(stderr, "%s: stat %s: %s\n", **argv, opt->mountpoint,
@@ -381,7 +355,7 @@ static hxmc_t *mtcr_slurp_file(const char *file)
  */
 static int mtcr_mount(struct mount_options *opt)
 {
-	const char *mount_args[9];
+	const char *mount_args[8];
 	const char *fsck_args[4];
 	struct stat sb;
 	int ret, argk;
@@ -481,9 +455,6 @@ static int mtcr_mount(struct mount_options *opt)
 
 	argk = 0;
 	mount_args[argk++] = "mount";
-#ifdef __linux__
-	mount_args[argk++] = "-n";
-#endif
 	if (opt->fstype != NULL) {
 		mount_args[argk++] = "-t";
 		mount_args[argk++] = opt->fstype;
@@ -535,17 +506,26 @@ static bool mtcr_get_umount_options(int *argc, const char ***argv,
 		HXOPT_AUTOHELP,
 		HXOPT_TABLEEND,
 	};
+	int ret;
 
 	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) <= 0)
 		return false;
 
+	pmtlog_path[PMTLOG_DBG][PMTLOG_STDERR] = Debug;
+
 	if (*argc < 2 || *(*argv)[1] == '\0') {
 		fprintf(stderr, "%s: You need to specify the container "
 		        "or its mountpoint\n", **argv);
 		return false;
 	}
 
-	opt->object = readlinkf((*argv)[1]);
+	ret = HX_realpath(&opt->object, (*argv)[1],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[1], strerror(-ret));
+		return false;
+	}
 	if (stat(opt->object, &sb) < 0) {
 		/* If it does not exist, it cannot be the container. */
 		opt->is_cont = false;
@@ -570,12 +550,12 @@ static bool mtcr_get_umount_options(int *argc, const char ***argv,
  */
 static int mtcr_remount(struct mount_options *opt)
 {
-	const char *rmt_args[6];
+	const char *rmt_args[5];
 	int ret, argk = 0;
-	char *mntpt;
+	char *mntpt, *cont;
 
 	ret = pmt_cmtab_get(opt->object, opt->is_cont ?
-	      CMTABF_CONTAINER : CMTABF_MOUNTPOINT, &mntpt, NULL, NULL, NULL);
+	      CMTABF_CONTAINER : CMTABF_MOUNTPOINT, &mntpt, &cont, NULL, NULL);
 	if (ret == 0) {
 		fprintf(stderr, "Nothing found that could be remounted.\n");
 		return 1;
@@ -584,10 +564,9 @@ static int mtcr_remount(struct mount_options *opt)
 		return ret;
 	}
 
+	if (!opt->no_update)
+		pmt_smtab_remove(mntpt, SMTABF_MOUNTPOINT);
 	rmt_args[argk++] = "mount";
-#ifdef __linux__
-	rmt_args[argk++] = "-i";
-#endif
 	rmt_args[argk++] = "-o";
 	rmt_args[argk++] = opt->extra_opts;
 	rmt_args[argk++] = mntpt;
@@ -599,10 +578,36 @@ static int mtcr_remount(struct mount_options *opt)
 		fprintf(stderr, "remount %s failed with run_sync status %d\n",
 		        opt->object, ret);
 
+	if (!opt->no_update)
+		pmt_smtab_add(cont, mntpt, "crypt", (opt->extra_opts != NULL) ?
+			opt->extra_opts : "defaults");
 	free(mntpt);
+	free(cont);
 	return ret;
 }
 
+/**
+ */
+static void mtcr_log_contents(const char *file)
+{
+	hxmc_t *ln = NULL;
+	FILE *fp;
+
+	if (file == NULL)
+		return;
+	w4rn("Dumping contents of %s\n", file);
+	if ((fp = fopen(file, "r")) == NULL) {
+		w4rn("Failed to open %s: %s\n", file, strerror(errno));
+		return;
+	}
+	while (HX_getl(&ln, fp) != NULL) {
+		HX_chomp(ln);
+		w4rn("%s\n", ln);
+	}
+	HXmc_free(ln);
+	fclose(fp);
+}
+
 /**
  * mtcr_umount - unloads the EHD from mountpoint
  *
@@ -610,7 +615,7 @@ static int mtcr_remount(struct mount_options *opt)
  */
 static int mtcr_umount(struct umount_options *opt)
 {
-	const char *umount_args[5];
+	const char *umount_args[3];
 	int final_ret, ret, argk = 0;
 	struct ehd_mount mount_info;
 	char *mountpoint = NULL;
@@ -623,16 +628,26 @@ static int mtcr_umount(struct umount_options *opt)
 		fprintf(stderr, "pmt_cmtab_get: %s\n", strerror(-ret));
 		return 0;
 	} else if (ret == 0) {
-		fprintf(stderr, "%s is not mounted (according to cmtab)\n",
-		        opt->object);
+		fprintf(stderr, "No vfsmount found while searching for \"%s\" "
+		        "as a container file, or as a mountpoint. (According "
+		        "to the intersection of cmtab (%s) with smtabs)\n",
+		        opt->object, pmt_cmtab_path());
+		mtcr_log_contents(pmt_cmtab_path());
+		mtcr_log_contents(pmt_smtab_path());
+		mtcr_log_contents(pmt_kmtab_path());
 		return 1;
+	} else {
+		if (ret & PMT_BY_CONTAINER)
+			w4rn("Found container in smtab\n");
+		if (ret & PMT_BY_CRYPTODEV)
+			w4rn("Found crypto device in smtab\n");
 	}
 
+	if (!opt->no_update)
+		pmt_smtab_remove(mountpoint, SMTABF_MOUNTPOINT);
+	pmt_cmtab_remove(mountpoint);
+
 	umount_args[argk++] = "umount";
-#ifdef __linux__
-	/* Always pass in -n, as we manually edit /etc/mtab */
-	umount_args[argk++] = "-ni";
-#endif
 	umount_args[argk++] = mountpoint;
 	umount_args[argk]   = NULL;
 
@@ -647,9 +662,6 @@ static int mtcr_umount(struct umount_options *opt)
 		fprintf(stderr, "ehd_unload: %s\n", strerror(-ret));
 		final_ret = 0;
 	} else {
-		if (!opt->no_update)
-			pmt_smtab_remove(mountpoint, SMTABF_MOUNTPOINT);
-		pmt_cmtab_remove(mountpoint);
 		final_ret = 1;
 	}
 
