@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2006 - 2009
+ *	Copyright © Jan Engelhardt, 2006 - 2011
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -212,6 +212,7 @@ static void volume_free(struct vol *vol)
 	free(vol->fstype);
 	free(vol->server);
 	free(vol->volume);
+	HXmc_free(vol->combopath);
 	free(vol->cipher);
 	free(vol->fs_key_cipher);
 	free(vol->fs_key_hash);
@@ -1262,14 +1263,11 @@ static const char *rc_volume(xmlNode *node, struct config *config,
 		vpt->fstype = xstrdup("auto");
 	}
 
-	if ((tmp = xml_getprop(node, "noroot")) != NULL) {
+	if ((tmp = xml_getprop(node, "noroot")) != NULL)
 		vpt->noroot = parse_bool_f(tmp);
-	} else if (vpt->fstype != NULL) {
+	else if (vpt->fstype != NULL)
 		/* Figure out whether we want to act as user. */
-		vpt->noroot =
-			strcmp(vpt->fstype, "fuse") == 0 ||
-			strcmp(vpt->fstype, "encfs13") == 0;
-	}
+		vpt->noroot = strcmp(vpt->fstype, "fuse") == 0;
 
 	/* Source location */
 	if ((tmp = xml_getprop(node, "server")) != NULL) {
@@ -1357,15 +1355,15 @@ static const char *rc_volume(xmlNode *node, struct config *config,
 
 //-----------------------------------------------------------------------------
 static const struct pmt_command default_command[] = {
-	{CMD_SMBMOUNT,   "smbfs", {"smbmount", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_SMBMOUNT,   "smbfs", {"smbmount", "%(COMBOPATH)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
 	{CMD_SMBUMOUNT,  "smbfs", {"smbumount", "%(MNTPT)", NULL}},
-	{CMD_CIFSMOUNT,   "cifs", {"mount", "-t", "cifs", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "user=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
-	{CMD_NCPMOUNT,   "ncpfs", {"ncpmount", "%(SERVER)/%(USER)", "%(MNTPT)", "-o", "pass-fd=0,volume=%(VOLUME)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_CIFSMOUNT,   "cifs", {"mount", "-t", "cifs", "%(COMBOPATH)", "%(MNTPT)", "-o", "user=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_NCPMOUNT,   "ncpfs", {"ncpmount", "%(COMBOPATH)", "%(MNTPT)", "-o", "pass-fd=0,volume=%(VOLUME)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
 	{CMD_NCPUMOUNT,  "ncpfs", {"ncpumount", "%(MNTPT)", NULL}},
 	{CMD_FUSEMOUNT,   "fuse", {"mount.fuse", "%(VOLUME)", "%(MNTPT)", "%(if %(OPTIONS),-o)", "%(OPTIONS)", NULL}},
 	{CMD_FUSEUMOUNT, "fuse",  {"fusermount", "-u", "%(MNTPT)", NULL}},
 	/* Do not use LCLMOUNT to avoid calling fsck */
-	{CMD_NFSMOUNT,    "nfs",   {"mount", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(SERVER):%(VOLUME)", "%(MNTPT)", NULL}},
+	{CMD_NFSMOUNT,    "nfs",   {"mount", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(COMBOPATH)", "%(MNTPT)", NULL}},
 	{CMD_NFSMOUNT,    "nfs4"},
 	{CMD_LCLMOUNT,    NULL,    {"mount", "-p0", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(VOLUME)", "%(MNTPT)", NULL}},
 	{CMD_CRYPTMOUNT,  "crypt", {"mount", "-t", "crypt", "%(if %(CIPHER),-ocipher=%(CIPHER))", "%(if %(FSKEYCIPHER),-ofsk_cipher=%(FSKEYCIPHER))", "%(if %(FSKEYHASH),-ofsk_hash=%(FSKEYHASH))", "%(if %(FSKEYPATH),-okeyfile=%(FSKEYPATH))", "%(if %(OPTIONS),-o%(OPTIONS))", "%(VOLUME)", "%(MNTPT)", NULL}},
